<?php
namespace app\common\model;

use think\Model;

/**
 * 登记记录表
 *
 * @author wj
 * @date 2022-07-22
 */
class appointmentmodel extends Model
{
    protected $table = "t_appointment";

    public function insertData($data)
    {
        if (!isset($data['oprdate']) || empty($data['oprdate']) || !is_string($data['oprdate'])) {
            $data['oprdate'] = date("Y-m-d H:i:s");
        }
        if (isset($data['id'])) {
            unset($data['id']);
        }
        $data = $this->formatData($data);
        if (empty($data)) {
            return false;
        }
        $id = $this->insertGetId($data);
        return $id ? $id : false;
    }

    /**
     * 校验入库数据
     * 20220119
     */
    private function formatData($data)
    {
        $useData = [];
        $fields = $this->getTableFields();
        foreach ($data as $key => $value) {
            if (in_array($key, $fields)) {
                $useData[$key] = $value;
            }
        }
        return $useData;
    }
    public function getInfo($where, $field = "*", $row = true)
    {
        $info = $this->field($field)->where($where);
        if ($row) {
            $info = $info->find();
        } else {
            $info = $info->select();
        }
        return empty($info) ? false : $info;
    }

    public function updateinfo($where, $updateData)
    {
        $row = $this->where($where)->update($updateData);
        return empty($row) ? false : $row;
    }

    public function deleteinfo($where)
    {
        $row = $this->where($where)->delete();
        return empty($row) ? false : $row;
    }

    public function getList($where = [], $field = "*", $page = 1, $size = 10, $order = "id desc", $group = "", $row = false)
    {
        $sqlObj = $this->where($where);
        if ("count" != $field) {
            if (empty($size)) {
                $sqlObj = $sqlObj->field($field)->order($order)->group($group)->page($page, $size);
            } else {
                $sqlObj = $sqlObj->field($field)->order($order)->group($group);
            }
            if ($row) {
                $data = $sqlObj->find();
            } else {
                $data = $sqlObj->select();
            }
        } else {
            $data = $sqlObj = $sqlObj->count();
        }
        return $data;
    }

    /**
     * 根据id获取信息
     * wj
     * 20220118
     */
    public function getinfobyid($id)
    {
        $where = ['id' => $id];
        $info = $this->where($where)->find();
        return $info;
    }
    /**
     * 根据code获取信息
     * wj
     * 20220118
     */
    public function getinfobycode($code)
    {
        $where = ['code' => $code];
        $info = $this->where($where)->find();
        return $info ? $info : false;
    }
    /**
     * 根据id修改数据
     * wj
     * 20220118
     */
    public function updatebyid($id, $updateData)
    {
        $where = ['id' => $id];
        $updateData = $this->formatData($updateData);
        if (empty($updateData)) {
            return false;
        }
        $row = $this->where($where)->update($updateData);
        return $row;
    }

    /**
     * 获取最后一条可用数据
     *
     * @param  [type] $wid
     * @return void
     * @author wj
     * @date 2022-07-22
     */
    public function getlastinfo($where)
    {
        $usewhere = ['isuse' => 0, 'ispay' => 1, 'oprdate' => ['>=', date("Y-m-d 00:00:00")]];
        if (isset($where['wid'])) {
            $usewhere['wid'] = $where['wid'];
        }
        if (isset($where['openid'])) {
            $usewhere['openid'] = $where['openid'];
        }
        if (isset($where['code'])) {
            $usewhere['code'] = $where['openid'];
        }
        if (isset($where['sfzid'])) {
            $usewhere['sfzid'] = $where['sfzid'];
        }
        $info = $this->where($usewhere)->find();
        return $info ? $info : false;
    }
    /**
     * 获取申请编码
     *
     * @return void
     * @author wj
     * @date 2022-07-22
     */
    public function createcode($str)
    {
        $code = substr($str, 0, 4) . date("YmdHis");
        return $code;
    }
}
